#!/usr/bin/perl -w
# convert 16x16 character image to binary and/or assembler charset
# (C) Arne Rossius 2012
#
# 2012-09-09  first version
# 2012-09-29  automatic calculation of char width and height from image size
# 2014-03-06  output C files

$hor = 1; # horizontal bytes?
$rev = 1; # set to 1 for MSB-left or MSB-top
$bin = 0; # enable binary file output
$asm = 1; # enable assembler file output
$c   = 0; # enable C file output

use Image::Magick;

if (! @ARGV) {
	die ("Usage: $0 <Filename.png> [<Filename.png> [...]]\n");
}

foreach $file (@ARGV) {
	if (! -f $file) {
		die ("Image file '$file' not found!\n");
	}
	
	# read image file
	$image = Image::Magick->new;
	$image->Read($file);
	$width = $image->Get('width');
	$height = $image->Get('height');
	print "Converting '$file' ($width x $height) ...\n";
	@pixels = $image->GetPixels(map=>'I', width=>$width, height=>$height, normalize=>1);
	
	# calculate and check char width and height
	$cw = $width / 16;
	$ch = $height / 16;
	if ($cw != int($cw)) { die "Image width must be a multiple of 16.\n"; }
	if ($ch != int($ch)) { die "Image height must be a multiple of 16.\n"; }
	if ($cw > 8) { die "Character width > 8 not supported.\n"; }
	print "  Char size: $cw x $ch\n";
	
	# open output file(s)
	$filename = $file;
	$filename =~ s/\.[A-Za-z0-9]+$//;
	if ($c) {
		open C, ">$filename.c" or die ("Failed to open C file.\n");
	}
	if ($asm) {
		open ASM, ">$filename.asm" or die ("Failed to open ASM file.\n");
	}
	if ($bin) {
		open BIN, ">$filename.bin" or die ("Failed to open BIN file.\n");
	}
	
	if ($hor) {
		$n_bytes = $ch;
		$n_bits = $cw;
	} else {
		$n_bytes = $cw;
		$n_bits = $ch;
	}
	
	if ($c) {
		$var = $filename;
		$var =~ s/^[0-9]|[^a-zA-Z0-9]/_/g;
		print C "#include <stdint.h>\n";
		print C "#include <avr/pgmspace.h>\n";
		print C "PROGMEM uint8_t ${var}[] = {\n";
	}

	# convert image to charset
	for ($y = 0; $y < 16; $y++) {
		for ($x = 0; $x < 16; $x++) {
			for ($byte = 0; $byte < $n_bytes; $byte++) {
				if ($c) {
					if (($byte == 0) and (($x % 2 == 0) or ($cw % 2 == 0))) {
						if ($y + $x > 0) {
							print C ",\n\t";
						} else {
							print C "\t";
						}
					} else {
						print C ", ";
					}
				}
				if ($asm) {
					if (($byte == 0) and (($x % 2 == 0) or ($cw % 2 == 0))) {
						print ASM "\n.db ";
					} else {
						print ASM ", ";
					}
				}
				$data = 0;
				for ($bit = 0; $bit < $n_bits; $bit++) {
					if ($hor) {
						$cx = $cw*$x + $bit;
						$cy = $ch*$y + $byte;
					} else {
						$cx = $cw*$x + $byte;
						$cy = $ch*$y + $bit;
					}
					if ($pixels[$cy * $width + $cx] < .5) {
						if ($rev) {
							$data |= 1<<(7-$bit);
						} else {
							$data |= 1<<$bit;
						}
					}
				}
				if ($c) {
					printf C "0x%02x", $data;
				}
				if ($asm) {
					printf ASM "0x%02x", $data;
				}
				if ($bin) {
					print BIN pack('C', $data);
				}
			}
		}
	}
	
	# close files
	if ($c) {
		print C "\n};\n";
		close C;
	}
	if ($asm) {
		print ASM "\n";
		close ASM;
	}
	if ($bin) {
		close BIN;
	}

}
